@@ -82,6 +82,30 @@ def generate_mappings():
     def exist_check(activity, xpath):
         return bool(rm_blank(activity.xpath(xpath)))
 
+    def check_value_is(activity, xpath, value, default):
+        # This is slightly odd. If there is no value provided,
+        # we should return what the expression would like us
+        # to say.
+        # e.g. if there is no aid type, then you might want
+        # to include this activity anyway, rather
+        # than excluding it.
+        if not exist_check(activity, xpath):
+            return default
+        if rm_blank(activity.xpath(xpath)[0]) == value:
+            return True
+        return False
+
+    def check_value_gte(activity, xpath, amount, default):
+        # This is slightly odd. If there is no value provided,
+        # we should return what the expression would like us
+        # to say.
+        # e.g. if there is no activity status, then you might 
+        # want to include this activity anyway, rather
+        # than excluding it.
+        if not bool(rm_blank(activity.xpath(xpath))):
+            return default
+        return bool(int(rm_blank(activity.xpath(xpath)[0])) >= int(amount))
+
     def exist_check_list(activity, xpath, codelist):
         outcome = False
         try:
@@ -154,4 +178,44 @@ def generate_mappings():
                 x_months_ago_check(activity, groups[3], groups[6]) or
                 x_months_ago_check(activity, groups[4], groups[6], groups[5]))
 
+    ## Conditional tests (only run if something)
+    
+    @add_partial('(\S*) exists\? \(if (\S*) is at least (\S*)\)') 
+    def exist_if_gte(activity, groups):
+        if check_value_gte(activity, groups[1], groups[2], True):
+            return exist_check(activity, groups[0])
+        else:
+            return None
+
+    @add_partial('(\S*) or (\S*) exists\? \(if (\S*) is at least (\S*)\)') 
+    def exist_or_if_gte(activity, groups):
+        if check_value_gte(activity, groups[2], groups[3], True):
+            return (exist_check(activity, groups[0]) or 
+                    exist_check(activity, groups[1]))
+        else:
+            return None
+
+    @add_partial('(\S*) exists\? \(if (\S*) is at least (\S*) and (\S*) is not (\S*)\)') 
+    def exist_if_both(activity, groups):
+        if (check_value_gte(activity, groups[1], groups[2], True) and not (check_value_is(activity, groups[3], groups[4], False))):
+            return exist_check(activity, groups[0])
+        else:
+            return None
+
+    @add_partial('(\S*) exists\? \(if (\S*) is at least (\S*) and \((\S*) or (\S*) is not (\S*)\)\)') 
+    def exist_if_both_or(activity, groups):
+        if (check_value_gte(activity, groups[1], groups[2], True) and not (check_value_is(activity, groups[3], groups[5], False) or check_value_is(activity, groups[4], groups[5], False))):
+            return exist_check(activity, groups[0])
+        else:
+            return None
+
+
+    @add_partial('(\S*) or (\S*) exists\? \(if (\S*) is at least (\S*) and \((\S*) or (\S*) is not (\S*)\)\)') 
+    def exist_or_if_both(activity, groups):
+        if (check_value_gte(activity, groups[2], groups[3], True) and not (check_value_is(activity, groups[4], groups[5], False) or check_value_is(activity, groups[4], groups[5], False))):
+            return (exist_check(activity, groups[0]) or 
+                    exist_check(activity, groups[1]))
+        else:
+            return None
+
     return mappings
